   /*-----------------------------------------------------------------*
   | The documentation and code below is supplied by The Methodology              
   | Center, Penn State University.             
   *------------------------------------------------------------------*                          
  /*------------------------------------------------------------------*
   | MACRO NAME  : %LCA_distal_LTB version 1.1.0
   |                     Note:  Replaces %LCA_distal version 3.1.0
   | SHORT DESC  : Estimate a distal outcome for a single or multiple 
   |               groups based on prior latent class membership
   | REQUIREMENTS: SAS v 9.1 or higher and PROC LCA v 1.2.6 or higher
   *------------------------------------------------------------------*
   | CREATED BY  : Jingyun Yang, Xianming Tan, and John Dziak  
   |               (May 9, 2016; re-released May 1, 2017)  
   | CONTACT : John Dziak (jjd264@psu.edu) or 
   |            Methodology Center Helpdesk (MChelpdesk@psu.edu)
   *------------------------------------------------------------------*
   | Copyright 2017, The Pennsylvania State University
   |
   | This software is distributed "as is" without any warranty.   
   | All warranties, including, but not limited to, implied 
   | warranties of merchantability and fitness for a particular.
   | purpose, are disclaimed.
   *------------------------------------------------------------------*
   |  Arguments in %LCA_distal_LTB
   |     input_data:          the input data set
   |     param:               the OUTPARAM file generated by PROC LCA
   |     distal:              the distal outcome variable in the dataset
   |     group:               the variable for multiple groups
   |     metric:              identifies the distal outcome variable 
   |                          as binary, continuous or count
   |     output_dataset_name: the filename for the macro's output 
   |     output_density:       the filename for conditional density when 
   |                          the distal outcome variable is continuous 
   |     plot_density:           whether the user wants to generate a 
   |                          conditional density plot (plot_density=1) 
   |                          or not (plot_density=0) 
   *--------------------------------------------------------------------*/
/* for model-based estimation 
    input: original dataset, _param, zc type;  
    output: Z|C, and plot data set for zc=2   */
%MACRO LCA_distal_LTB(  input_data = , /* input data set  */
                    param= ,      /* input beta parameters from LCA */
                    distal= ,     /* distal outcome variable */
                    group= ,       /*the variable for multiple groups*/
                    metric= 1,       /* This tells SAS distal z is 
                                         1: binary, or 
                                         2: continuous, 
                                         3: count, or 
                                         4: categorical  */             
                    output_dataset_name= res1, /* output results  */
                    out_density= cond_density,  /* name of the density 
                                                   dataset for continuous 
                                                  distal outcomes*/
                    plot_density= 0,
                    alpha= .05, /* optional 1-coverage for confidence intervals */
                    covariance_beta = , /* optional name of the covariance matrix 
                                          dataset to calculate confidence intervals */
                    weight =  /* optional weighting variable; must be the same as 
                                  was used in the analysis */
                  ); 
    %IF %TRIM("&weight")="" %THEN %DO;
        %LET there_are_weights = 0;
    %END; %ELSE %DO;
        %LET there_are_weights = 1; 
    %END;     
    %IF %EVAL((&there_are_weights) & (&metric ^= 1)) %THEN %DO;
        %PUT Error: Weights are currently only supported for binary distal LCA.;
        PROC IML;
             PRINT("Error: Weights are currently only supported for binary distal LCA.");
        QUIT;
        %ABORT;
    %END;
    %IF %TRIM("&group")="" %THEN %DO;   /*If there is no group variable*/
        %LET there_are_groups = 0;
        %LET group = _GroupForMacro_;
        DATA _sortedinput;
            SET &input_data;
            _GroupForMacro_ = 1;
        RUN;
        %distal_ltb_nogroup( the_data = _sortedinput,
                         param = &param,
                         distal = &distal,
                         metric = &metric,
                         weight = &weight,
                         output_dataset_name = &output_dataset_name,
                         out_density = &out_density );                                
        %IF &metric=2 AND &plot_density=1 %THEN %DO;  
                         /*if the user want to plot the density data*/
            %cplot(&out_density,0);
        %END;
    %END; %ELSE %DO;   /*If there are multiple groups*/
        %LET there_are_groups = 1;
        PROC SQL;
                RESET NOPRINT;
                SELECT unique group into :all_group separated by ' '
                FROM &param;
                RESET PRINT;
                QUIT;  
        PROC SORT DATA=&input_data OUT=_sortedinput;
            BY &group;
        RUN;   * sort the input data by group;
        %LET group_num =%EVAL(%SYSFUNC(count(%cmpres(&all_group),%STR( )))+1);
                   * the total number of groups;
        %IF &metric=1 OR &metric=3 %THEN %DO;    
            %DO i=1 %TO &group_num;
                DATA _sub_input_group&i;
                    SET _sortedinput;
                    IF &group=&i;
                RUN;
                DATA _sub_param_group&i;
                    SET &param;
                    IF group=&i;
                RUN;
                %distal_ltb_nogroup(the_data = _sub_input_group&i,
                                param = _sub_param_group&i,
                                distal = &distal,
                                metric = &metric,
                                weight = &weight,
                                output_dataset_name = _bcc_check_group&i,
                                out_density = &out_density );
                DATA _bcc_temp_group&i;
                    SET _bcc_check_group&i;
                    &group=&i;
                RUN;
            %END;
            DATA &output_dataset_name;
                ARRAY fixorder &group;
                SET _bcc_temp_group1-_bcc_temp_group&group_num;
            RUN;
        %END;
        %ELSE %IF &metric=2 %THEN %DO;  /* For continuous case, treated separately 
                                           to create a dataset for conditional density*/
            %DO i=1 %TO &group_num;
                DATA _sub_input_group&i;
                    SET _sortedinput;
                    IF &group=&i;
                RUN;
                DATA _sub_param_group&i;
                    SET &param;
                    IF group=&i;
                RUN;
                %distal_ltb_nogroup(the_data = _sub_input_group&i,
                                param = _sub_param_group&i,
                                distal = &distal,
                                metric = &metric,
                                weight = ,
                                output_dataset_name = _bcc_check_group&i,
                                out_density = _bcc_density_group&i );
                DATA _bcc_temp_group&i;
                    SET _bcc_check_group&i;
                    &group=&i;
                RUN;
                DATA _temp_density_group&i;
                    SET _bcc_density_group&i;
                    &group=&i;;
                RUN;
            %END;
            DATA &output_dataset_name;
                ARRAY fixorder &group;
                SET _bcc_temp_group1-_bcc_temp_group&group_num;
            RUN;
            DATA &out_density;
                ARRAY fixorder &group;
                SET _temp_density_group1-_temp_density_group&group_num;
            RUN;  /*Create the dataset for conditional density*/
        %END;
        %IF &metric=2 AND &plot_density=1 %THEN %DO;  /*if the user wants to plot the density data*/
            %DO j=1 %TO &group_num;
                DATA _temp_plot_den;
                    SET _temp_density_group&j(drop=&group);
                RUN;
                %cplot(_temp_plot_den,&j);
            %END;
        %END;
        %IF &metric=4 %THEN %DO;  /*If the distal outcome is categorical (but not just binary)*/
            PROC SQL;
                RESET NOPRINT;
                SELECT unique &distal into :all_z separated by ' '
                FROM _sortedinput;
                RESET PRINT;
            QUIT;  
            %LET categn =%EVAL(%SYSFUNC(count(%CMPRES(&all_z),%STR( )))+1);* the total number of groups;
            %LET group_tot=%EVAL(&group_num/&categn);
            %DO i=1 %TO &group_tot;
                DATA _cat_subparam_group&i;
                    SET &param;
                    IF group<=%EVAL(&i*&categn) AND group>%EVAL((&i-1)*&categn);
                RUN; /*get the sub_data for parameter for each group after running LCA*/
                DATA _sub_input_group&i;
                    SET _sortedinput;
                    IF &group<=%EVAL(&i*&categn) AND &group>%EVAL((&i-1)*&categn);
                RUN;  /*get the sub input data for each group*/
                %distal_ltb_nogroup(the_data = _sub_input_group&i,
                                param = _cat_subparam_group&i,
                                distal = &distal,
                                metric = &metric,
                                weight = &weight,
                                output_dataset_name = _categ_temp_group&i,
                                out_density = &out_density );
                DATA _categ_temp_group&i;
                    SET _categ_temp_group&i;
                    &group=&i;
                RUN;
            %END;
            %LET org_group=%SUBSTR(&group,5);
            DATA &output_dataset_name(rename=(&group=&org_group));
                ARRAY fixorder &group;
                SET _categ_temp_group1-_categ_temp_group&group_tot;
                LABEL &group="&org_group";
            RUN;
        %END;
    %END;
    /************ Print the Estimates ***************/ 
    PROC IML;
        USE &output_dataset_name;
           READ ALL INTO results;
        CLOSE &output_dataset_name;
        FILE PRINT;
        PUT "Class-Specific Distribution Estimates for &distal";
        %IF &metric=1 %THEN %DO;
            %IF %EVAL(&there_are_groups) %THEN %DO;  
                ngroups = NROW(results)/2;
                nclasses = NCOL(results)-3; 
                DO group = 1 TO ngroups;
                    L = "Within &group = " + CHAR(group,3) + ":";
                    maindata = (results[((group-1)*2+(1:2)),3:NCOL(results)])`;
                    r = "Overall" // "Class" + CHAR((1:nclasses)`,2);
                    c = "P(Z=0)" || "P(Z=1)";
                    PRINT maindata[rowname=r colname=c format=10.4 label=L];
                END;
                PUT "P(Z=k) in the table above is the probability that &distal equals k.";
            %END; %ELSE %DO;
                nclasses = NCOL(results)-2; 
                maindata = (results[,2:NCOL(results)])`;
                r = "Overall" // "Class" + CHAR((1:nclasses)`,2);
                c = "P(Z=0)" || "P(Z=1)";
                PRINT maindata[rowname=r colname=c format=10.4 label="Estimated Probabilities"];
                PUT "P(Z=k) in the table above is the probability that &distal equals k.";
            %END;
        %END;     
        %IF %EVAL(&metric=2) %THEN %DO;
            %IF %EVAL(&there_are_groups) %THEN %DO;   
                ngroups = NROW(results)/2;
                DO group = 1 TO ngroups;       
                    PUT "Within group" group " on &distal:";
                    Class = (1:(NCOL(results)-2))`; 
                    Mean = (results[(group-1)*2+1,3:NCOL(results)])`;
                    Mode = (results[(group-1)*2+2,3:NCOL(results)])`;
                    PRINT Class Mean[format=10.4] Mode[format=10.4];
                END;      
            %END; %ELSE %DO;         
                Class = (1:(NCOL(results)-1))`; 
                Mean = (results[1,2:NCOL(results)])`;
                Mode = (results[2,2:NCOL(results)])`;
                PRINT Class Mean[format=10.4] Mode[format=10.4];
            %END;
        %END;     
        %IF %EVAL(&metric = 3) %THEN %DO;
            %IF %EVAL(&there_are_groups) %THEN %DO;   
                ngroups = NROW(results)/2;
                DO group = 1 TO ngroups;       
                    PUT "Within group" group " on &distal:";
                    Class = (1:(NCOL(results)-2))`; 
                    Mean = (results[(group-1)*2+1,3:NCOL(results)])`;
                    Mode = (results[(group-1)*2+2,3:NCOL(results)])`;
                    PRINT Class Mean[format=10.4] Mode[format=10.0];
                END;      
            %END; %ELSE %DO;         
                Class = (1:(NCOL(results)-1))`; 
                Mean = (results[1,2:NCOL(results)])`;
                Mode = (results[2,2:NCOL(results)])`;
                PRINT Class Mean[format=10.4] Mode[format=10.0];
            %END;
        %END;     
        %IF %EVAL(&metric=4) %THEN %DO;
            %IF %EVAL(&there_are_groups) %THEN %DO;        
                nclasses = NCOL(results)-3; 
                c = "Overall" || "Class" + CHAR((1:nclasses),2);
                r = "Group" + CHAR(results[,1],3) + ", Outcome" + CHAR(results[,2],3);
                maindata=results[,3:NCOL(results)]; 
                PRINT maindata[colname=c rowname = r format=10.4 label="Estimated Probabilities"];  
            %END; %ELSE %DO;
                nlevels = NROW(results);
                nclasses = NCOL(results)-2;
                maindata = (results[,2:NCOL(results)])`;
                r = "Overall" // "Class" + CHAR((1:nclasses)`,2);
                c = "P(Z=" + CHAR((1:nlevels)`,1) + ")";
                PRINT maindata[rowname=r colname=c format=10.4 label="Estimated Probabilities"]; 
                PUT "P(Z=k) in the table above is the probability that &distal equals k.";
            %END;
        %END;     
    RUN;QUIT; 
    /************ Print the Standard Errors and Tests ***************/
    %IF &metric=1 %THEN %DO; 
        %IF %TRIM("&covariance_beta")="" %THEN %DO;
            %PUT No covariance matrix was specified -- will not calculate standard errors;  
        %END; %ELSE %DO;
            %binary_conf_int_ltb( the_data = _sortedinput,
                                  param=&param,
                                  distal=&distal,
                                  group=&group,
                                  weight=&weight,
                                  covariance_beta=&covariance_beta,
                                  alpha=&alpha);  
        %END;
    %END;
    PROC DATASETS NOLIST;
        DELETE _: ;
    RUN;QUIT;
%MEND LCA_distal_LTB;
 
%MACRO distal_ltb_nogroup(
                the_data = ,    /* input data set  */
                param = ,      /* input beta parameters from LCA */
                distal = ,     /* distal outcome variable */
                metric = ,            /* This tells SAS distal z is 1: binary, 
                              or 2: continuous, or 3: count, or 4: categorical  */ 
                weight = ,   /* optional name of weighting variable */ 
                output_dataset_name =    , /* output results  */
                out_density = );
    %IF %TRIM("&weight")="" %THEN %DO;
        %LET there_are_weights = 0;
    %END; %ELSE %DO;
        %LET there_are_weights = 1; 
    %END;     
    %IF %EVAL((&there_are_weights) & (&metric ^= 1)) %THEN %DO;
        %PUT Error: Weights are currently only supported for the binary case.;
        PROC IML;
             PRINT("Error: Weights are currently only supported for the binary case.");
        QUIT;
        %ABORT;
    %END; 
    /* for discrete Z(binary, count, or categorical), find P(Z=z) */ 
    %IF &metric =1 or &metric = 3 or &metric = 4 %THEN %DO;  /* discrete z
                   (binary, count, or categorical) */
        PROC FREQ DATA=&the_data NOPRINT; 
            %IF %EVAL(&there_are_weights=1) %THEN %DO;
                WEIGHTS &weight;
            %END;
            TABLES &distal/OUT=_freq; 
        RUN;
        DATA _freq; 
            SET _freq;
            prob = percent/100;
            KEEP &distal prob;
        RUN;
    %END;
    /* continuous Z, find density estimates */
    %IF &metric =2 %THEN %DO;  /* continuous z */
        ODS LISTING CLOSE;
        PROC KDE data= &the_data;
               UNIVAR &distal /OUT = _freq ; 
        RUN;
        ODS LISTING;
        DATA _freq; 
            SET _freq;
            &distal = value;
            prob = density;
            KEEP &distal prob;
        RUN;
    %END;
    /* clean beta parameter data set */
    DATA _param_;
        SET &param; 
        IF (param ^= 'RHO'); *('GAMMA', 'BETA');  
        KEEP ESTLC:;
    RUN;
    /* given _param and _freq, we can estimate Z|C using method 1 */
    PROC IML; 
        USE _freq;
            READ ALL INTO freq;
        CLOSE _freq;
        USE _param_;
            READ ALL INTO param;
        CLOSE _param_;
        /* deal with . ( due to reference ) in beta parameter */
        /* e.g., if =. then =0, or simply replace . with 0 */
        IF (&metric =4) THEN DO;  /* categorical Z */
            ZC_joint  = freq[,2] # param;  
                          /* Pr(Z=z) * Pr(C=c|Z=z) ; */
            C_margin = J(1, NROW(freq), 1) * ZC_joint;   
                          /* Pr(C=c) = sum_z Pr(Z=z) * Pr(C=c|Z=z) ; */
            kk= J(1, NROW(freq), 1);
            tmp = ( ZC_joint / C_margin); 
                          /* Pr(Z=z) * Pr(C=c|Z=z) / Pr(C=c); */
        END;
        ELSE DO; /* binary, continuous, or count Z */
            param[LOC(param = .)] = 0;
            exp_part  = exp( ( J(NROW(freq),1, 1) || freq[, 1] )*param[2:3, ] );  
                          /* exp(b_0k + b_1k*z ) ; */
            den_part = (exp_part*J(NCOL(param),1, 1) ) *param[1,];  
                          /* P(C=k)*( 1+ sum_k exp(b_0k + b_1k*z ) ) ; */
            tmp = (freq[,2]#(exp_part/den_part)); 
                          /* P(z)* exp(b_0k + b_1k*z ) / 
                              P(C=k)*( 1+ sum_k exp(b_0k + b_1k*z ) ) ; */
        END; 
        RES = freq || tmp; 
        names = ('Z_C' + CHAR( (1:ncol(param)),2) ); 
        CALL CHANGE(names," ","0",0);
        cols = 'Z' || 'Density_Z' || names ; 
        IF (&metric =1 | &metric = 4) THEN DO;  /* binary or categorical Z */
            CREATE &output_dataset_name FROM RES [ COLNAME = cols ];
                APPEND FROM RES;
            CLOSE &output_dataset_name;
        END; 
        IF (&metric =2) THEN DO;  /* plot data set  */
            CREATE &out_density FROM RES [ COLNAME = cols ];
                APPEND FROM RES;
            CLOSE &out_density;
        END; 
        IF (&metric =2 | &metric = 3) THEN DO;  /* continuous Z */
            ress = J(2,ncol(param)+1, 0);
            ress[1,1] = 1; ress[2,1] = 2;
            /* calculate conditional mean and mode or median  */
            idx = tmp[<:>, ]; /* for mode */ 
            c_sum = tmp; 
            DO i = 2 TO nrow(tmp);
                c_sum[i, ] = c_sum[i, ] + c_sum[i-1, ]; 
            END;
            abs_sum = abs(c_sum - 0.5); 
            idx2 = abs_sum[>:<, ]; /* for median */ 
            del = 1; 
            IF (&metric = 2) THEN 
                del = (MAX(freq[,1]) - MIN(freq[,1]))/(nrow(freq)); 
                /* check if freq[, 1] is equally spaced */ 
            DO k = 1 TO ncol(param);            
                ress[1, k+1] = sum( freq[,1]#tmp[, K])*del; *mean; 
                ress[2, k+1] = freq[idx[k] ,1]; * mode ; 
            END;
            names = ('Z_C' + CHAR( (1:ncol(param)),2) ); 
            CALL CHANGE(names," ","0",0);
            cols = 'M_Mode' || names ; 
            CREATE &output_dataset_name FROM ress [ COLNAME = cols ];
                APPEND  FROM ress;
            CLOSE &output_dataset_name;
        END;  
    QUIT;
%MEND distal_ltb_nogroup;

%MACRO cplot(densityplot_data,gp); /*The macro for creating density plots*/
    %IF %EVAL(&there_are_groups=0) %THEN %DO;
        PROC CONTENTS DATA=&densityplot_data OUT=_tmp_density NOPRINT;    
        RUN;
        PROC SQL;
        /* find out the number of varaibles, reduced from B-splines */
            RESET NOPRINT;
            SELECT (MAX(VARNUM)) INTO :aa FROM _tmp_density;  
        QUIT;
        %LET size = %TRIM(&aa);
        %LET latent_num =%EVAL(&size-2);* the total number of groups;
        %PUT latent class number is &latent_num, size=&size;
        %MACRO SP;
            DATA _crp_&distal;
                SET &densityplot_data (KEEP=Z density_Z);
                CLASS="density_Z";
            RUN;
               %DO i=1 %TO &latent_num;                                                   
                DATA _crp_group&i(RENAME=(Z_C0&i=density_Z));
                    SET &out_density(KEEP=Z Z_C0&i);
                    CLASS="Z_C0&i";
                RUN;
            %END;
            DATA _crp_comb;
                SET _crp_&distal _crp_group1-_crp_group&latent_num;
            RUN;
        %MEND SP;
        %SP;   /*Reshape the data to create a new grouping variable for plot*/
        PROC TEMPLATE;
            DEFINE STATGRAPH mygraph_tvem;
                BEGINGRAPH;   
                    ENTRYTITLE "Conditional Density Plot";                                                 
                    LAYOUT lattice / columndatarange=union rowweights=(0.9 0.1);            
                        LAYOUT overlay /yaxisopts=(label="Density");  
                             SERIESPLOT x=Z y=density_Z/group=class name="sp"; 
                            DISCRETELEGEND "sp" / title="Legend";
                        ENDLAYOUT;
                    ENDLAYOUT; 
                ENDGRAPH;     
            END;          
        RUN; 
        PROC SGRENDER DATA=_crp_comb TEMPLATE=mygraph_tvem;                                   
        RUN; 
    %END;
    %ELSE %DO; /*if there are multiple groups*/
        PROC CONTENTS DATA=&densityplot_data OUT=_tmp_density NOPRINT;    
        RUN;
        PROC SQL;
        /* find out the number of varaibles, reduced from B-splines */
            RESET NOPRINT;
            SELECT (MAX(VARNUM)) INTO :aa FROM _tmp_density;  
        QUIT;
        %LET size = %TRIM(&aa);
        %LET latent_num =%EVAL(&size-2);* the total number of groups;
        %PUT latent class number is &latent_num size=&size;
        %MACRO SP;
            DATA _crp_Z;
                SET &densityplot_data (keep=Z density_Z);
                CLASS="density_Z";
            RUN;
               %DO i=1 %TO &latent_num;                                                   
                DATA _crp_group&i(keep=Z Z_C0&i class rename=(Z_C0&i=density_Z));
                SET &out_density;
                IF &group=&gp;
                CLASS="Z_C0&i";
                RUN;
            %END;
            DATA _crp_comb;
                SET _crp_Z _crp_group1-_crp_group&latent_num;
            RUN;
        %MEND SP;
        %SP;   /*Reshape the data to create a new grouping variable for plot*/
        PROC TEMPLATE;  
              DEFINE statgraph mygraph_tvem;                                                 
                  BEGINGRAPH;   
                      ENTRYTITLE "Conditional Density Plot for group= &gp";                                                 
                      LAYOUT lattice / COLUMNDATARANGE=union ROWWEIGHTS=(0.9 0.1);            
                          LAYOUT overlay /yaxisopts=(label="Density");  
                             SERIESPLOT x=Z y=density_Z/group=class name="sp"; 
                             DISCRETELEGEND "sp" / title="Legend";
                          ENDLAYOUT;                                                   
                    ENDLAYOUT; 
                  ENDGRAPH;     
             END;          
        RUN; 
        PROC SGRENDER DATA=_crp_comb TEMPLATE=mygraph_tvem;                                   
        RUN; 
    %END;
%MEND cplot;

%MACRO binary_conf_int_ltb(the_data = ,
                           param = ,
                           distal = ,
                           group = ,
                           weight = ,  
                           covariance_beta = ,
                           alpha = ); 
    %IF %TRIM("&weight")="" %THEN %DO;
        %LET there_are_weights = 0;
    %END; %ELSE %DO;
        %LET there_are_weights = 1; 
    %END;     
    /********** Determine the number of groups ************ */
    PROC IML;  
        USE &param;
            READ ALL VAR {group} INTO group;
        CLOSE &param;
        num_groups = group[<>];  /* If there's no grouping variable, this will just be 1. */
        CALL SYMPUT("num_groups",CHAR(num_groups));
    QUIT;
    /********** Loop over groups ************ */
    %DO g = 1 %TO %EVAL(&num_groups); 
        ODS SELECT NONE;
        %IF %EVAL(&there_are_weights = 0) %THEN %DO;
            PROC LOGISTIC DATA=&the_data;
                WHERE &group=&g;
                MODEL &distal(DESCENDING) = ;
                ODS OUTPUT ParameterEstimates=_ParameterEstimates;
            RUN;
        %END; %ELSE %DO;
            PROC SURVEYLOGISTIC DATA=&the_data;
                MODEL &distal(DESCENDING) = ;
                WHERE &group=&g;
                WEIGHT &weight;
                ODS OUTPUT ParameterEstimates=_ParameterEstimates;
            RUN;
        %END;
        ODS SELECT ALL;
        DATA _ParameterEstimates; SET _ParameterEstimates;
            CALL SYMPUT("omega",Estimate);
            CALL SYMPUT("se_omega",StdErr);
        RUN;
        PROC IML; 
            START SafeExp(x);
                SafeX = x >< 30;
                y = EXP(SafeX); 
                RETURN(y);
            FINISH SafeExp; 
            g = &g; 
            /*********** Get gamma for each class ******************************* */ 
            USE &param; 
                READ ALL WHERE((param={"GAMMA"})&(group=g)) INTO gamma; 
            CLOSE &param; 
            gamma = gamma[,3:NCOL(gamma)];
                       /* leave out columns for group number and response category */ 
            nclass = NCOL(gamma);
            /*********** Get omega and var(omega) ******************************** */
            omega = &omega;
            pi = SafeExp(omega)/(1+SafeExp(omega));
            var_omega = &se_omega**2;
            /*********** Get beta and the reference class ************************ */
            USE &param; 
                READ ALL WHERE((param={"BETA"})&(group=g)) into beta; 
            CLOSE &param;  
            beta = beta[,3:ncol(beta)];
                       /* leave out columns for group number and response category */ 
            refclass = LOC(beta[1,]=.);
            IF (NCOL(refclass)^=1) THEN DO;
                PRINT "error:  missing coefficient estimates";
            END;
            beta[,refclass] = 0; /* reference betas are zero by convention */
            /*********** Get beta0 and beta1 ********************* */
            beta0 = beta[1,];  /* intercept */
            beta1 = beta[2,];   /* slope or effect */
            /*********** Get gamma_given_0 and gamma_given_1 ********************* */
            gamma_given_0 = SafeExp(beta0)/(SafeExp(beta0)[+]);
            gamma_given_1 = SafeExp(beta0+beta1)/(SafeExp(beta0+beta1)[+]);
            /*********** Get omega_given_class ************************************* */
            omega_given_class = omega + LOG(1e-30 + gamma_given_1) - LOG(1e-30 + gamma_given_0);
            omega_given_class = omega_given_class`; /* express as column vector */
            /*********** Get the cov(beta) **************************************** */
            USE &covariance_beta;
                READ ALL INTO cov_beta_all_groups;
            CLOSE &covariance_beta;
            p = NCOL(cov_beta_all_groups);
            cov_beta = cov_beta_all_groups[(((g-1)*p+1):(g*p)),1:p];  
            /*********** Get the inside matrix ************************************ */
            temp1 = ( var_omega     ||   J(1,p,0)  );
            temp2 = ( J(p,1,0)      ||   cov_beta  );  
            inner_matrix = temp1 // temp2;
            /*********** Get the jacobian ***************************************** */
            jacobian = j(nclass,2*(nclass-1)+1,0);
            DO row_class = 1 TO nclass;
                col_for_writing = 1;
                col_class_not_counting_ref = 0; 
                jacobian[col_for_writing,1] = 1;
                DO col_class_counting_ref = 1 TO nclass;
                    IF (col_class_counting_ref ^= refclass) THEN DO;
                        col_class_not_counting_ref = col_class_not_counting_ref + 1; 
                        delta = (row_class=col_class_counting_ref);
                        /* Jacobian for beta0 */
                        col_for_writing = col_for_writing + 1; 
                        jacobian[row_class,col_for_writing] = gamma[row_class] *
                                                 (delta - gamma[col_class_counting_ref]) *
                                             (gamma_given_0[row_class] -
                                              gamma_given_1[row_class]) /
                                             (gamma_given_0[row_class] *
                                              gamma_given_1[row_class]);
                        /* Jacobian for beta1 */
                        col_for_writing = col_for_writing + 1; 
                        jacobian[row_class,col_for_writing] = gamma[row_class] *
                                                 (delta - gamma[col_class_counting_ref]) /
                                                    gamma_given_1[row_class];   
                    END;
                END;
            END;
            /*********** Calculate standard errors **************************** */
            cov_omega_given_class = jacobian * inner_matrix * jacobian`;
            std_errs_omega_given_class = J(nclass,1,0);
            DO k = 1 TO nclass;
                std_errs_omega_given_class[k] = SQRT(cov_omega_given_class[k,k]);
            END;
            /*********** Calculate confidence intervals *********************** */
            omega_given_class_upper = omega_given_class + 
                                       QUANTILE('NORMAL',1-.5*&alpha) *
                                       std_errs_omega_given_class;
            omega_given_class_best = omega_given_class;
            omega_given_class_lower = omega_given_class - 
                                       QUANTILE('NORMAL',1-.5*&alpha) * 
                                       std_errs_omega_given_class;
            pi_given_class_upper = SafeExp(omega_given_class_upper) / 
                                    (1+SafeExp(omega_given_class_upper));
            pi_given_class_best = SafeExp(omega_given_class_best) / 
                                    (1+SafeExp(omega_given_class_best));
            pi_given_class_lower = SafeExp(omega_given_class_lower) / 
                                    (1+SafeExp(omega_given_class_lower));
            /*********** Calculate test statistics *********************** */
            wald = J(nclass,nclass,.);  /* wald[i,j] for i=/=j will be the Wald 
                                           statistic for testing whether omega for
                                           class i and omega for class j are the same */
            wald_pvalue = J(nclass,nclass,.);  /* wald_pvalue[i,j] will be the p-value for the test; */ 
            IF (nclass>1) THEN DO;
                DO row_class = 1 TO nclass;
                    DO colclass = (row_class+1) TO nclass;
                        wald[row_class,colclass] = ((omega_given_class_best[row_class]-
                                                    omega_given_class_best[colclass])**2)/
                                                  (cov_omega_given_class[row_class,row_class]+
                                                   cov_omega_given_class[colclass,colclass]-
                                                   2*cov_omega_given_class[row_class,colclass]);
                        wald[colclass,row_class] = wald[row_class,colclass];
                        wald_pvalue[row_class,colclass] = 1-probchi(wald[row_class,colclass],1);
                        wald_pvalue[colclass,row_class] = wald_pvalue[row_class,colclass];
                    END;
                END; 
            END;
            /******************** Output the data ************************ */
            class = 1:NROW(pi_given_class_best);
            CREATE _omega_by_class_this_group VAR { 
                omega_given_class_best
                std_errs_omega_given_class
                class  
            }; APPEND;
            CLOSE _omega_by_class_this_group;
            CREATE _pi_by_class_this_group VAR { 
                pi_given_class_best
                pi_given_class_upper
                pi_given_class_lower
                class
            }; APPEND;
            CLOSE _pi_by_class_this_group;
            CREATE _wald_teststats_this_group FROM wald;
                APPEND FROM wald;
            CLOSE _wald_teststats_this_group;
            CREATE _wald_pvalues_this_group FROM wald_pvalue; 
                APPEND FROM wald_pvalue;
            CLOSE _wald_pvalues_this_group; 
            FILE PRINT;
            num_groups = &num_groups;
            IF (num_groups = 1) THEN DO; 
                PUT "Estimated Class-Conditional Probabilities for P(&distal=1|Class=c)";
            END; ELSE DO;
                PUT "----- Group &g on Variable &Group -------------------";
                PUT "Estimated Class-Conditional Probabilities for P(&distal=1|Class=c,&group = &g)";
            END;
            PUT "                                        Confidence Interval ";
            PUT "Class      Probability                  For Probability";
            DO k = 1 TO nclass;
                point = pi_given_class_best[k];
                lower = pi_given_class_lower[k];
                upper = pi_given_class_upper[k];
                PUT k 5.0
                    '          '
                    point 8.4 
                    '             ( ' 
                    lower 8.4 
                    ', ' 
                    upper 8.4 ' )';   
            END; 
            df = 1;
            PUT " Wald Test on Class-Conditional Probabilities";
            PUT "   Classes            Wald    df  p-value";
            IF (nclass>1) THEN DO;
                DO k1 = 1 TO nclass;
                    DO k2 = (k1+1) TO nclass;
                        teststat = wald[k1,k2];
                        pval = wald_pvalue[k1,k2];
                        PUT k1 3.0 
                            '  vs. '
                            k2 3.0
                            ' '
                            teststat 14.4
                            '  '
                            df  3.0
                            pval 8.4;
                    END;
                END;
            END;
        QUIT;
        DATA _omega_by_class_this_group; SET _omega_by_class_this_group; &group = &g; RUN;
        DATA _pi_by_class_this_group; SET _pi_by_class_this_group; &group = &g; RUN;
        DATA _wald_teststats_this_group; SET _wald_teststats_this_group; &group = &g; RUN;
        DATA _wald_pvalues_this_group; SET _wald_pvalues_this_group; &group = &g; RUN;
        %IF %EVAL(&g = 1) %THEN %DO;
            DATA _omega_by_class; SET _omega_by_class_this_group; RUN;
            DATA pi_by_class; SET _pi_by_class_this_group; RUN;
            DATA wald_teststats; SET _wald_teststats_this_group; RUN;
            DATA wald_pvalues; SET _wald_pvalues_this_group; RUN;
        %END; %ELSE %DO; 
            DATA _omega_by_class; SET _omega_by_class _omega_by_class_this_group; RUN;
            DATA pi_by_class; SET pi_by_class _pi_by_class_this_group; RUN;
            DATA wald_teststats; SET wald_teststats _wald_teststats_this_group; RUN;
            DATA wald_pvalues; SET wald_pvalues _wald_pvalues_this_group; RUN;
        %END; 
        %IF %EVAL(&num_groups = 1) %THEN %DO;
            DATA pi_by_class; SET pi_by_class; DROP _GroupForMacro_; RUN;
            DATA wald_teststats; SET wald_teststats; DROP _GroupForMacro_; RUN;
            DATA wald_pvalues; SET wald_pvalues; DROP _GroupForMacro_; RUN;
        %END;
    %END;
%MEND binary_conf_int_ltb;
